using System;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.XR;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using Object = UnityEngine.Object;
using System.Text.RegularExpressions;

namespace Cotyounoyume
{
    public class AddonClothSimItem : MonoBehaviour 
    {
		public Atom atom;
		public EBATCore core;
		public AddonManager1 addon1;
		
		public ClothesStateChanger csc;
		public bool isUndressActive = false;
		
		public string[] ngwords = {
			"eye", "nose", "face", "mouth", 
			"hair", "brow",
			"drip", "makeup",
			"mtk2", 
		};

		public AddonClothSimItem(EBATCore core){
			atom = core.atom;
			this.core = core;
		}
		
		public void LoadSettings(){
			try{
				this.addon1 = core.addonManager1;
			}catch (Exception e){
				core.LoggerE(e);
			}
		}
				
		public void ApplySettings(){
			BackupClothesState();
		}

		public void Manage(float time){
		}	
		
		public void OnDestroy(){

		}
		
		public void OnOrgasmStart()
		{
		}
		
		public void OnOrgasmStop()
		{

		}

		public void CheckClothAttribute(){
			csc.CheckClothAttribute();
		}
		
		public void BackupClothesState(){
			try{
				if(csc != null){
					csc.ChangeClothesIfNeeded();
					SetRemoveOptions();
					csc.ReloadStatus();
//					csc.OnDestroy();
				}else{
					csc = new ClothesStateChanger(core.atom, core, ngwords);
					SetRemoveOptions();
					csc.ReloadStatus();
				}
			}catch(Exception e){
				core.LoggerE(e);
			}			
		}
		
		public void SetRemoveOptions(){
			if(!core.IsLoaded()) return; 
			
			csc.SetRemoveOptions(
				core.addonManager1.holdNGWordsOnGlobal,
				core.addonManager1.holdClothSimOnGlobal, 
				core.addonManager1.holdSkinWrapOnGlobal, 
				core.addonManager1.wrapAlterToSimOnGlobal);
		}
		
		public void ChangeClothes(){
			csc.ChangeClothesIfNeeded();
		}
		
		public void Break(){
			isUndressActive = true;
			csc.InitialChange("Break");
		}
		
		public void Break2(){
			isUndressActive = true;
			csc.InitialChange("Break2");
		}
		
		public void UndressAndRemove(){
			core.StartCoroutine(UndressAndRemoveCoroutine());
		}
		
		public IEnumerator UndressAndRemoveCoroutine(){
			Undress("UndressMiddle");
			yield return new WaitForSeconds(0.05f);
			csc.ReloadStatus();
			yield return new WaitForSeconds(0.05f);
			RemoveClothes();
			yield return null;
		}
		
		public void Undress(string type){
			isUndressActive = true;
			core.EmbarrassALittle();
			if(type == "Undress"){
				core.StartCoroutine(UndressCoroutine(1.0f, 5, false, type));
			}else if(type == "UndressMiddle"){
				core.StartCoroutine(UndressCoroutine(1.0f, 5, false, type));
			}else if(type == "UndressForce"){
				core.StartCoroutine(UndressCoroutine(4.0f,15, false, type));
			}else if(type == "UndressHalf"){
				core.StartCoroutine(UndressCoroutine(0.7f, 5, false));
			}else if(type == "Reverse"){
				core.StartCoroutine(ReverseGravityCoroutine(2.0f, 12, false, false));
			}else if(type == "UndressAndReverse"){
				core.StartCoroutine(ReverseGravityCoroutine(2.0f, 12, false, true));				
			}else if(type == "UndressAndReverseForce"){
				core.StartCoroutine(ReverseGravityCoroutine(4.0f, 12, false, true, isForce:true));				
			}else{
				core.Info("In Undress: Unknown Command: " + type);
			}
		}
		
		public void UndressStop(){
			try{
				isUndressActive = false;
				csc.UndressStop();
			}catch(Exception e){
				core.LoggerE(e);
			}			
		}
		
		public void AlterSkinWrap(){
			try{
				csc.WrapAlter();			
			}catch(Exception e){
				core.LoggerE(e);
			}						
		}
		
		public void WrapNormal(){
			try{
				csc.WrapNormal();			
			}catch(Exception e){
				core.LoggerE(e);
			}						
		}
		
		public void RemoveClothes(){
			try{
				csc.RemoveClothes();
			}catch(Exception e){
				core.LoggerE(e);
			}						
		}
		
		public void RemoveResetClothes(){
			try{
				csc.RemoveResetClothes();
			}catch(Exception e){
				core.LoggerE(e);
			}						
		}
		
		public void RecoverClothesState(){
			try{
				isUndressActive = false;
				csc.Recover();
			}catch(Exception e){
				core.LoggerE(e);
			}			
		}		
		
		public void ResetClothesState(){
			try{
				csc.Reset();
			}catch(Exception e){
				core.LoggerE(e);
			}			
		}		

		public IEnumerator UndressCoroutine(float power, int seconds, bool enableStopping = false, string undressType = "Undress"){
//			if(wrapChangeOnGlobal){ csc.WrapAlter(); }
			
			csc.InitialChange(undressType);
			yield return new WaitForSeconds(0.1f);
					
			int num = 10 * seconds;
			float ratio;
			for(int i = 0; i < num; i++){
				ratio = (float)((float)(num - i) / ((float)num)) * power;
				if(isUndressActive){
					csc.Change(undressType, ratio);
					yield return new WaitForSeconds(0.1f);
				}
			}
			if(enableStopping){
				csc.Change(undressType, 0f);
			}
			isUndressActive = false;
		}
		
		public IEnumerator ReverseGravityCoroutine(float power, int seconds, bool enableStopping = false, bool enableUndress = false, bool isForce = false){
//			if(wrapChangeOnGlobal){ csc.WrapAlter(); }

			string undressType = (enableUndress) ? "UndressAndReverse" : "Reverse";
			if(isForce){
				undressType = "UndressAndReverseForce";
			}
			csc.InitialChange(undressType);
			yield return new WaitForSeconds(0.1f);
			
			int num = 10 * seconds;
			float ratio;
			for(int i = 0; i < num; i++){
				ratio = (float)((float)(num - i) / ((float)num)) * power;
				ratio *= UnityEngine.Random.Range(0.5f, 1.0f);
				if(isUndressActive){
					csc.Change(undressType, ratio);						
					yield return new WaitForSeconds(0.1f);
				}
			}
			if(enableStopping){
				csc.Change("Reverse", 0f);
			}
			isUndressActive = false;
		}
	}
}